Package: mclink
Title: Metabolic Pathway Completeness and Abundance Calculation
Version: 1.1.2
Authors@R: 
    person(given = "Liuyang",
           family = "Li",
           email = "cyanobacteria@yeah.net",
           role = c("aut", "cre"),
           comment = c(ORCID = "0000-0001-6004-9437"))
Description: 
    Provides tools for analyzing metabolic pathway completeness, abundance, and transcripts
    using KEGG Orthology (KO) data from (meta)genomic and (meta)transcriptomic studies.
    Supports both completeness (presence/absence) and abundance-weighted analyses.
    Includes built-in KEGG reference datasets. For more details see Li et al. (2023) <doi:10.1038/s41467-023-42193-7>.
URL: https://github.com/LiuyangLee/mclink
BugReports: https://github.com/LiuyangLee/mclink/issues
License: GPL-3
Encoding: UTF-8
RoxygenNote: 7.3.2
Imports: data.table (>= 1.17.8), dplyr (>= 1.1.4), stringr (>= 1.5.1),
        tibble (>= 3.2.1), utils
Depends: R (>= 3.5)
LazyData: false
Config/check/use_internal_tz: TRUE
NeedsCompilation: no
Packaged: 2026-07-20 07:41:24 UTC; Administrator
Author: Liuyang Li [aut, cre] (ORCID: <https://orcid.org/0000-0001-6004-9437>)
Maintainer: Liuyang Li <cyanobacteria@yeah.net>
Repository: CRAN
Date/Publication: 2026-07-20 08:00:02 UTC

# Generated by roxygen2: do not edit by hand

export(KO_Sample_wide)
export(KO_pathway_ref)
export(add_rows_if_not_exists)
export(ata_cal)
export(convert_abundance_to_presence)
export(create_sub_module_sample)
export(escape_special_chars)
export(extract_inner_brackets)
export(group_ko_by_module)
export(mclink)
export(merge_module_name)
export(process_all_modules)
export(process_all_pathways)
export(process_module_brackets)
export(process_module_definition)
export(process_module_loop_comma)
export(process_module_loop_plu_comma)
export(process_module_loop_plus)
export(process_module_step)
export(process_module_structure)
export(process_step_comma)
export(process_step_direct)
export(process_step_plus)
export(process_step_space)
export(read_and_process_KO_table)
export(read_and_process_pathway_infor)
export(remove_outer_brackets)
importFrom(dplyr,"%>%")
importFrom(utils,data)
importFrom(utils,packageVersion)

#' @title Read and Process KO Sample Table with Pathway Information
#'
#' @description This function reads a KO sample table (wide format), validates its contents against
#' pathway information, and returns a filtered table containing only KOs present in both datasets.
#'
#' @param in_KO_Sample_wide Character string specifying the path to the input KO sample table file.
#'        Should be a tab-delimited file with KO identifiers as row names.
#' @param pathway_infor Data frame containing pathway information, see examples.
#'
#' @return A list with log and a data frame.The data frame contains the filtered sample data, with only rows that match KOs in the
#'         pathway information. Includes an added 'Orthology_Entry' column containing the row names.
#'
#' @export
read_and_process_KO_table <- function(in_KO_Sample_wide, pathway_infor) {
  # Use tryCatch function to handle potential errors
  tryCatch({
    # Read the file
    # Sample_KO = read.csv(in_KO_Sample_wide, sep='\t',header = TRUE, row.names = 1, check.names = F)
    # Sample_KO = data.table::fread(in_KO_Sample_wide, sep = "\t", header = TRUE, check.names = FALSE, data.table = FALSE)
    Sample_KO = in_KO_Sample_wide
    rownames(Sample_KO) = Sample_KO[,1]
    Sample_KO = Sample_KO[,-1, drop = FALSE]

    timestamp <- function() format(Sys.time(), "[%Y-%m-%d %H:%M:%S]")
    log_entry <- function(msg) {
      paste0(timestamp(), " ", msg)
    }

    # Check if the file was successfully imported
    if (!is.null(Sample_KO)) {
      message(paste0('[',format(Sys.time(), "%Y-%m-%d %H:%M:%S"),'] Genome-KO File successfully imported.'))
      KO_log <- list(
          log_entry("Genome-KO File successfully imported.")
        )
    } else {
      KO_log <- list(
          log_entry("*** Genome-KO File import failed, please check the input dataframe format ***")
        )
      stop(paste0('[',format(Sys.time(), "%Y-%m-%d %H:%M:%S"),'] *** Genome-KO File import failed, please check the input dataframe format ***\n'))
    }

    KO_list <- unique(pathway_infor$Orthology_Entry)

    # Check the intersection of the first column and KO_list
    common_genes <- intersect(rownames(Sample_KO), KO_list)

    if (length(common_genes) > 0) {
      message(paste0('[',format(Sys.time(), "%Y-%m-%d %H:%M:%S"),'] There are ',length(common_genes),' intersect KOs between the KO list and input dataframe.'))
      KO_log <- c(KO_log, list(
          log_entry(paste0('There are ',length(common_genes),' intersect KOs between the KO list and input dataframe: ', paste(common_genes, collapse = ' ')))
        ))
    } else {
      KO_log <- c(KO_log, list(
          log_entry('*** There is no intersection between the first column of the input dataframe and KO list, please check the input dataframe format ***')
        ))
      stop(paste0('[',format(Sys.time(), "%Y-%m-%d %H:%M:%S"),'] *** There is no intersection between the first column of the input dataframe and KO list, please check the input dataframe format ***\n'))
    }

    # Use the mutate function to add a new column Orthology_Entry, the value of which is the row names of the data frame
    Sample_KO <- dplyr::mutate(Sample_KO, Orthology_Entry = rownames(Sample_KO))
    # Filter out the rows where Orthology_Entry is in pathway_infor$Orthology_Entry
    Sample_KO <- Sample_KO[rownames(Sample_KO) %in% unique(pathway_infor$Orthology_Entry), , drop = F]

    return(list(data = Sample_KO, log = KO_log))
  }, error = function(e) {
    stop(paste0('[',format(Sys.time(), "%Y-%m-%d %H:%M:%S"),'] *** File import failed, please check the input dataframe format ***\n'))
  })
}

#' @title Remove Outer Parentheses from String
#'
#' @description This function checks if a string is wrapped in outer parentheses
#'              and removes them if present.
#'
#' @param s A character string to be processed
#'
#' @return The input string with outer parentheses removed (if they existed),
#'         or the original string if no outer parentheses were found.
#' @export
remove_outer_brackets <- function(s) {
  # Check if the string starts with '(' and ends with ')' using regex
  if (base::grepl("^\\(.*\\)$", s)) {
    # If outer parentheses exist, remove them using regex substitution
    # ^ matches start of string, $ matches end
    # \\( and \\) escape the parentheses
    # (.*) captures everything inside the parentheses
    s <- base::sub("^\\((.*)\\)$", "\\1", s)
  }
  return(s)
}

# R/zzz.R

.onLoad <- function(libname, pkgname) {
  # 1. 初始化包选项（保持不变）
  op <- options()
  op.mclink <- list(
    mclink.path = tempdir(),
    mclink.verbose = FALSE
  )
  toset <- !(names(op.mclink) %in% names(op))
  if (any(toset)) options(op.mclink[toset])

  # 2. 安全加载数据（关键改进）
  ns <- asNamespace(pkgname)
  tryCatch({
    utils::data(list = c("KO_pathway_ref", "KO_Sample_wide"),
                package = pkgname,
                envir = ns)
  }, error = function(e) {
    warning("Data load failed: ", e$message, call. = FALSE)
  })

  invisible()
}

.onAttach <- function(libname, pkgname) {
  # 使用完全限定名调用（关键修复）
  version <- utils::packageVersion(pkgname)
  packageStartupMessage("Welcome to mclink v", version)
}

#' @importFrom dplyr %>%
NULL
#' @importFrom utils packageVersion data
#' @export KO_pathway_ref
#' @export KO_Sample_wide
NULL  #
#' Metabolic Pathway Coverage Analysis
#'
#' @description
#' Analyzes metabolic pathway completeness/abundance from (meta)genome KO presence/abundance data.
#' Can use either built-in KEGG datasets or user-provided data frames. Output includes pathway coverage
#' metrics and detected KOs in each pathway/module.
#'
#' The distill analysis of KEGG Module coverage is calculated based on the abundance or presence of KOs
#' in a given module, as per the KEGG Module Definition. In detail, the coverage of a KEGG Module is
#' determined by first dividing a set of KOs into distinct steps. The coverage for each step is then
#' calculated separately, and summarized as the coverage of this KEGG Module. When calculating coverage,
#' spaces or plus signs connecting KO numbers are interpreted as AND operators, while commas are
#' interpreted as OR operators. For instance, to calculate the completeness for the
#' module M00020: “K00058 K00831 (K01079, K02203, K22305)”:
#' 1. Convert the abundance table of KOs into a 0-1 matrix.
#' 2. Consider K00058 as step 1, K00831 as step 2, and (K01079, K02203, K22305) as step 3.
#' 3. Calculate the `maximum` (or `minimum`, `mean`) value of step 3 (K01079, K02203, K22305).
#'    If presence of any KO indicates completeness, calculate the `maximum` value;
#'    If all KOs must be resent for completeness, calculate the `minimum` value.
#'    For a moderate approach, calculate the `mean` value.
#' 4. Use this value along with the values of step 1 and step 2 to calculate the average value,
#'    representing the completeness of the module M00020.
#' @param ref Pathway information data frame. When `NULL` (default),
#'        uses the built-in \code{\link{KO_pathway_ref}} dataset. Must contain the
#'        same columns as the built-in dataset if providing custom data.
#' @param data Sample KO abundance data frame. When `NULL` (default),
#'        uses the built-in \code{\link{KO_Sample_wide}} dataset.
#' @param table_feature Analysis type, either:
#'        \itemize{
#'          \item "completeness" (binary presence/absence, default)
#'          \item "abundance" (weighted by KO abundance)
#'        }
#' @param plus_scale_method Scaling method for plus-separated KOs (K1+K2+...) (Enzyme subunits/Protein complexes):
#'        \itemize{
#'          \item "mean" - Moderate approach (default), calculates average value of all components
#'          \item "min" - Rigorous/conservative estimate, uses lowest value (all components must be present)
#'          \item "max" - Liberal estimate, uses highest value (any component indicates completeness)
#'        }
#' @param comma_scale_method Scaling method for comma-separated KOs (K1,K2,...) (Gene isoforms/Alternative pathways):
#'        \itemize{
#'          \item "max" - For completeness analysis (default), any component indicates functional pathway
#'          \item "sum" - For abundance analysis, sums all functionally equivalent variants
#'        }
#' @param out_dir Output directory path. If `NULL` (default), results are only
#'        returned as R objects without writing files.
#' @param split_by_pathway Logical. If `TRUE`, splits results by pathway/module.
#'        Requires non-NULL `out_dir`. Default: `FALSE`.
#'
#' @return A list containing:
#' \itemize{
#'   \item coverage - Data frame with pathway coverage metrics
#'   \item detected_KOs - List of detected KOs per pathway/module
#'   \item log - log of the analysis process
#' }
#'
#' If `out_dir` is specified, results are also written as TSV files.
#'
#' @export
#'
#' @examples
#' data(KO_pathway_ref)
#' data(KO_Sample_wide)
#' selected_modules <- c("M00176","M00165","M00173","M00374","M00375","M00376","M00377")
#' KO_pathway_ref_selected <- KO_pathway_ref[KO_pathway_ref$Module_Entry %in% selected_modules, ]
#' mc_list =
#'   mclink(ref = KO_pathway_ref_selected,
#'          data = KO_Sample_wide,
#'          table_feature = "completeness",
#'          plus_scale_method = "min",
#'          comma_scale_method = "max")
#' mc_coverage = mc_list[["coverage"]]
#' mc_detected_KOs = mc_list[["detected_KOs"]]
#' mc_log = mc_list[["log"]]
#' print(head(mc_coverage))
mclink <- function(ref = NULL,
                   data = NULL,
                   table_feature = "completeness",
                   plus_scale_method = "mean",
                   comma_scale_method = "max",
                   out_dir = NULL,
                   split_by_pathway = FALSE) {

  # Load default datasets if not provided
  if (is.null(ref)) {
    utils::data("KO_pathway_ref", envir = environment())
    ref <- KO_pathway_ref
  }

  if (is.null(data)) {
    utils::data("KO_Sample_wide", envir = environment())
    data <- KO_Sample_wide
  }

  # Input validation
  stopifnot(
    is.data.frame(ref),
    is.data.frame(data),
    table_feature %in% c("completeness", "abundance"),
    plus_scale_method %in% c("mean", "min", "max"),
    comma_scale_method %in% c("max", "sum")
  )

  message(paste0('[', format(Sys.time(), "%Y-%m-%d %H:%M:%S"), ']', ' mclink started!'))
  message(paste0('[',format(Sys.time(), "%Y-%m-%d %H:%M:%S"),'] '),'Input Sample-KO table type: ',table_feature)
  message(paste0('[',format(Sys.time(), "%Y-%m-%d %H:%M:%S"),'] '),'Scale method for plus: ',plus_scale_method)
  message(paste0('[',format(Sys.time(), "%Y-%m-%d %H:%M:%S"),'] '),'Scale method for comma: ',comma_scale_method)
  timestamp <- function() format(Sys.time(), "[%Y-%m-%d %H:%M:%S]")
  log_entry <- function(msg) {
    paste0(timestamp(), " ", msg)
  }
  global_log <- list(
      log_entry("mclink started!"),
      log_entry(paste0('Input Sample-KO table type: ',table_feature)),
      log_entry(paste0('Scale method for plus: ',plus_scale_method)),
      log_entry(paste0('Scale method for comma: ',comma_scale_method))
    )
  ##########################      pathway infor input      ##########################
  # 'MASH_KEGG_Energy_metabolism.20240315.tsv'
  pathway_infor_list = read_and_process_pathway_infor(ref)
  pathway_infor = pathway_infor_list[['data']]
  pathway_log = pathway_infor_list[['log']]
  module_level <- pathway_infor %>%
    dplyr::select(Module_Entry, Level_2, Level_3, Module_Name, Definition) %>%
    {unique(.)}
  global_log <- c(global_log, pathway_log)
  #message(paste0('[',format(Sys.time(), "%Y-%m-%d %H:%M:%S"),']'),'dim(module_level)...,',dim(module_level))
  #message(head(module_level))
  ##########################      Sample-KO depth input      ##########################
  Sample_KO_list = read_and_process_KO_table(data, pathway_infor)
  Sample_KO_abundance = Sample_KO_list[['data']]
  Sample_KO_log = Sample_KO_list[['log']]
  global_log <- c(global_log, Sample_KO_log)
  ##############    convert presence     ##############
  if (table_feature == "completeness") {
    message(paste0('[',format(Sys.time(), "%Y-%m-%d %H:%M:%S"),'] '),'Converting abundance table to completeness table...')
    global_log <- c(global_log, list(
        log_entry('Converting abundance table to completeness table...')
      ))
    Sample_KO_abundance = convert_abundance_to_presence(Sample_KO_abundance)
  }
  ##############    Calculating coverage of respective modules     ##############
  module_table_list = process_all_modules(pathway_infor, Sample_KO = Sample_KO_abundance,
                                          plus_scale_method,comma_scale_method,verbose = F)
  module_table_coverage = module_table_list[["data"]]
  module_table_log = module_table_list[["log"]]
  Module_Sample_coverage = merge_module_name(pathway_infor, module_table = module_table_coverage)
  global_log <- c(global_log, module_table_log)
  ##############    Output present KOs of respective modules    ##############
  message(paste0('[',format(Sys.time(), "%Y-%m-%d %H:%M:%S"),'] '),'Summarizing present KOs of respective modules...')
  global_log <- c(global_log, list(
      log_entry('Summarizing present KOs of respective modules...')
    ))
  Module_Sample_KO_list = group_ko_by_module(pathway_infor, Sample_KO_abundance) %>%
    {.[match(rownames(Module_Sample_coverage), rownames(.)), match(colnames(Module_Sample_coverage), colnames(.)), drop = FALSE]}
  mc_detected_KOs = Module_Sample_KO_list %>%
    {tibble::rownames_to_column(., var = "Module_Name")} %>%
    {base::merge(module_level, ., by = "Module_Name", all.x = T)} %>%
    dplyr::arrange(match(Module_Name, module_level$Module_Name)) %>%
    dplyr::select(c(Module_Entry,Level_2,Level_3,Module_Name, Definition), dplyr::everything())
  ##############    Output coverage of respective modules for all pathways    ##############
  mc_coverage = Module_Sample_coverage %>%
    {tibble::rownames_to_column(., var = "Module_Name")} %>%
    {base::merge(module_level, ., by = "Module_Name", all.x = T)} %>%
    dplyr::arrange(match(Module_Name, module_level$Module_Name)) %>%
    dplyr::select(c(Module_Entry,Level_2,Level_3,Module_Name, Definition), dplyr::everything())

  global_log <- c(global_log, list(log_entry('mclink finished!')))

  if (!is.null(out_dir)) {
    if (!dir.exists(out_dir)) {
      dir.create(out_dir, recursive = TRUE)
    }

    if (table_feature == "completeness") {
      # output detected KOs and coverage
      out_file <- base::file.path(out_dir, paste0('mc_detected_KOs', '.tsv'))
      data.table::fwrite(mc_detected_KOs, file = out_file, sep = "\t", quote = FALSE)

      out_file <- base::file.path(out_dir, paste0('mc_completeness', '.tsv'))
      data.table::fwrite(mc_coverage, file = out_file, sep = "\t", quote = FALSE)

      out_file <- base::file.path(out_dir, paste0('mc_completeness', '.log'))
      writeLines(as.character(unlist(global_log)), con = out_file)

      if (split_by_pathway) {
        out_dir_ko_sample_by_pathway_presence <- base::file.path(out_dir, 'mc_by_pathway')
        if (!dir.exists(out_dir_ko_sample_by_pathway_presence)) {
          dir.create(out_dir_ko_sample_by_pathway_presence, recursive = TRUE)
        }

        mc_coverage_arrange <- mc_coverage %>%
          dplyr::select(-c(Module_Entry, Level_2, Level_3, Definition)) %>%
          tibble::column_to_rownames(var = "Module_Name")

        Module_Sample_pathway_presence <- process_all_pathways(pathway_infor,
                                                               mc_coverage_arrange,
                                                               out_dir_ko_sample_by_pathway_presence,
                                                               compare_method = "round",
                                                               plus_scale_method,
                                                               comma_scale_method)
      }
    } else if (table_feature == "abundance") {
      # output detected KOs and coverage
      out_file <- base::file.path(out_dir, paste0('mc_detected_KOs','.tsv'))
      data.table::fwrite(mc_detected_KOs, file = out_file, sep = "\t", quote = FALSE)

      out_file <- base::file.path(out_dir, paste0('mc_abundance','.tsv'))
      data.table::fwrite(mc_coverage, file = out_file, sep = "\t", quote = FALSE)

      out_file <- base::file.path(out_dir, paste0('mc_abundance', '.log'))
      writeLines(as.character(unlist(global_log)), con = out_file)

      if (split_by_pathway) {
        out_dir_ko_sample_by_pathway_log <- base::file.path(out_dir, 'log_scale_by_pathway')
        if (!dir.exists(out_dir_ko_sample_by_pathway_log)) {
          dir.create(out_dir_ko_sample_by_pathway_log, recursive = TRUE)
        }

        out_dir_ko_sample_by_pathway_avg <- base::file.path(out_dir, 'avg_scale_by_pathway')
        if (!dir.exists(out_dir_ko_sample_by_pathway_avg)) {
          dir.create(out_dir_ko_sample_by_pathway_avg, recursive = TRUE)
        }

        mc_coverage_arrange <- mc_coverage %>%
          dplyr::select(-c(Module_Entry, Level_2, Level_3, Definition)) %>%
          tibble::column_to_rownames(var = "Module_Name")

        Module_Sample_pathway_log <- process_all_pathways(pathway_infor,
                                                          mc_coverage_arrange,
                                                          out_dir_ko_sample_by_pathway_log,
                                                          compare_method = "log",
                                                          plus_scale_method,
                                                          comma_scale_method)

        Module_Sample_pathway_avg <- process_all_pathways(pathway_infor,
                                                          mc_coverage_arrange,
                                                          out_dir_ko_sample_by_pathway_avg,
                                                          compare_method = "avg",
                                                          plus_scale_method,
                                                          comma_scale_method)
      }
    }
  }
  message(paste0('[', format(Sys.time(), "%Y-%m-%d %H:%M:%S"), ']', ' mclink finished!'))
  mc_list <- list(coverage = mc_coverage, detected_KOs = mc_detected_KOs, log = unlist(global_log))
  return(mc_list)
}

#' @title Process Comma-Separated KOs with Specified Scaling Method
#'
#' @description Handles comma-separated KOs by applying the specified scaling method (sum or max).
#'              Processes multiple KOs separated by commas and aggregates them into a single row.
#'
#' @param module_abundance Data frame containing KO abundance data with required columns:
#'                         Orthology_Entry, Module_Entry, Definition
#' @param KOs Character vector of comma-separated KO IDs (default: "K14126,K14127,K14128")
#' @param aggregrate_rowname Base name for row aggregation (default: 'step_1')
#' @param step_count Processing step counter (default: 1)
#' @param comma_scale_method Scaling method for comma-separated KOs ("sum" or "max")
#'
#' @return List containing:
#'         - abundance_table: Processed data with aggregated values
#'         - step_count: Updated step counter
#'         - abundance_log: log
#' @export
process_step_comma <- function(module_abundance, KOs = c("K14126,K14127,K14128"), aggregrate_rowname,
                               step_count = 1, comma_scale_method) {
  # Process comma-separated KOs
  # For comma-separated entries, add new_step_name = paste0(aggregrate_rowname, '_', step_count)
  KOs_scale <- base::strsplit(KOs, ",")[[1]]
  #cat(paste0('\t\tRunning KOs comma: ',aggregrate_rowname," = ",KOs_scale))
  log_messages <- list(paste0('[',format(Sys.time(), "%Y-%m-%d %H:%M:%S"),']','    ','Running KOs comma: ',aggregrate_rowname," = ",KOs_scale))
  # Prepare abundance table with selected KOs
  abundance_table = module_abundance %>%
    {
      rownames(.) = (.$Orthology_Entry)
      (.)
    } %>%
    {dplyr::select(., -c(Orthology_Entry, Module_Entry, Definition))} %>%
    {add_rows_if_not_exists(., add_rows = KOs_scale)} %>%
    {.[rownames(.) %in% KOs_scale, , drop = F]}

  # Apply specified scaling method
  if (comma_scale_method == "sum") {
    abundance_table_scale = abundance_table %>% {t(colSums(.))}
  } else if (comma_scale_method == "max") {
    abundance_table_scale = abundance_table %>% {t(apply(., 2, max))}
  } else {
    stop(paste("Unknown comma scale method:", comma_scale_method))
  }

  # Format final result
  abundance_table = abundance_table_scale %>%
    {as.data.frame((.), row.names = aggregrate_rowname)} %>%
    {dplyr::mutate(.,
                   Orthology_Entry = rownames(.),
                   Module_Entry = unique(module_abundance$Module_Entry),
                   Definition = unique(module_abundance$Definition)
    )}

  step_count = step_count + 1
  return(list(abundance_table = abundance_table, step_count = step_count, abundance_log = log_messages))
}

#' @title Calculate Log2 Ratio of Sample Values to Row Means
#'
#' @description This function calculates the log2 ratio of each value in a data frame
#' to its corresponding row mean. Zero values are preserved as zeros in the output.
#'
#' @param data A data frame or matrix containing numerical values to be processed.
#'             Rows represent features (e.g., KO terms) and columns represent samples.
#'             Default is KO_Sample_table.
#'
#' @return A data frame of the same dimensions as input, where each value is:
#'         - log2(sample_value/row_mean) when both sample_value and row_mean are non-zero
#'         - 0 when either sample_value or row_mean is zero
#'         The Mean_RA column used for calculations is removed from the output.
#'
#' @export
ata_cal = function(data = KO_Sample_table){
  data$Mean_RA = base::apply(data, 1, mean)
  scaled_df = data
  for(col in colnames(data)){
    scaled_df[[col]] <- ifelse(((data[[col]] != 0) & (data$Mean_RA != 0)), log2(data[[col]]/data$Mean_RA), 0)
  }
  scaled_df = dplyr::select(scaled_df, -Mean_RA)
  return(scaled_df)
}

#' @title Process All Pathways Analysis
#'
#' @description Processes module sample data across all pathways with specified scaling methods.
#'              Handles different comparison methods and outputs results by pathway.
#' @param pathway_infor Data frame containing pathway information, see examples.
#' @param Module_Sample Data frame of module sample data to process
#' @param out_DIR_Module_Sample_by_pathway Output directory for pathway-specific results
#' @param compare_method Comparison method to use: "log" (log10 transform),
#'                       "avg" (average calculation), or "round" (simple rounding)
#' @param plus_scale_method Scaling method for plus-separated KOs ("mean", "min", or "max")
#' @param comma_scale_method Scaling method for comma-separated KOs ("sum" or "max")
#'
#' @return Main outputs are written to:
#'         - Combined module file (All_modules.*.tsv)
#'         - Pathway-specific files (via create_sub_module_sample)
#' @export
process_all_pathways <- function(pathway_infor, Module_Sample, out_DIR_Module_Sample_by_pathway, compare_method = c("log", "avg", "round"),
                                 plus_scale_method,comma_scale_method) {
  if (compare_method == "log") {
    Module_Sample_scale <- Module_Sample %>%
      {round(log10((.) + 1), 4)}
  } else if (compare_method == "avg") {
    Module_Sample_scale <- Module_Sample %>%
      {round(ata_cal(.), 4)}
  } else if (compare_method == "round") {
    Module_Sample_scale <- Module_Sample %>%
      {round(., 4)}
  } else {
    stop("Invalid compare_method. Choose anyone of the 'log', 'avg' or 'round'.")
  }

  out_file <- base::file.path(out_DIR_Module_Sample_by_pathway, paste0('All_modules',".tsv"))

  Module_Sample_out = as.data.frame(Module_Sample_scale) %>%
    {tibble::rownames_to_column(., var = "Module_Name")}

  data.table::fwrite(Module_Sample_out, file = out_file, sep = "\t", quote = FALSE)

  create_sub_module_sample(pathway_infor, Module_Sample_scale, out_DIR_Module_Sample_by_pathway, plus_scale_method,comma_scale_method)
}

#' @title Process Module Definition String
#'
#' @description Cleans and processes module definition strings by removing various patterns and formatting
#'              elements to extract core KO relationships. Handles special cases like negative KOs and
#'              parenthetical expressions.
#'
#' @param sub_Sample_KO_pathway Data frame containing module definitions in a 'Definition' column
#'
#' @return A list with log and character vector of cleaned module definition strings. The vector contains:
#'         - Removed negative KO indicators
#'         - Simplified parentheses
#'         - Normalized space
#' @export
process_module_definition <- function(sub_Sample_KO_pathway) {
  # Use unique() to remove duplicate values in Definition column
  unique_definitions <- unique(sub_Sample_KO_pathway$Definition)
  # Use gsub() to remove specific patterns from strings:
  # Remove "-- " pattern
  step1 <- gsub("-- ", "", unique_definitions)
  # Remove " --" pattern
  step2 <- gsub(" --", "", step1)
  # Remove "-Knumber" pattern (negative KO indicators)
  step3 <- gsub("-K\\d+", "", step2)
  # Handle negative parenthetical case: M00011 -(K00242,K18859,K18860)
  step4 <- gsub("-\\(.*?\\)", "", step3)
  # Replace multiple consecutive spaces with single space
  step5 <- gsub("\\s+", " ", step4)
  # Replace "(Knumber)" pattern with "Knumber" (simplify parentheses)
  result <- gsub("\\((K\\d+)\\)", "\\1", step5)
  #cat(paste0('\nProcessing module steps: ', unique_definitions, '\n'))
  #cat(paste0('After omitting minus KOs: ', result, '\n'))

  timestamp <- function() format(Sys.time(), "[%Y-%m-%d %H:%M:%S]")
  log_entry <- function(msg) {
    paste0(timestamp(), " ", msg)
  }
  log_messages <- list(
      log_entry(paste0('Processing module steps: ', unique_definitions)),
      log_entry(paste0('After omitting minus KOs: ', result))
    )
  return(list(
    definition = result,
    log = log_messages
  ))
}

#' @title Convert Abundance Values to Presence/Absence Indicators
#'
#' @description Transforms a numeric abundance matrix into a binary presence/absence matrix,
#'              where 1 indicates presence (abundance > 0) and 0 indicates absence.
#'              Preserves row names as Orthology_Entry column in the output.
#'
#' @param module_abundance A data frame containing KO abundance data, must include:
#'                         - Rows named by Orthology_Entry (KO identifiers)
#'                         - Numeric columns representing sample abundances
#'                         - An Orthology_Entry column
#'
#' @return A data frame with:
#'         - Binary values (1 = present, 0 = absent) for each sample
#'         - Original row names preserved in Orthology_Entry column
#'         - Same dimensions as input (excluding the Orthology_Entry column)
#' @export
convert_abundance_to_presence <- function(module_abundance) {
  module_abundance %>%
    {
      rownames(.) = (.$Orthology_Entry)
      (.)
    } %>%
    {dplyr::select(., -c(Orthology_Entry))} %>%
    # {apply(., 2, function(x) ifelse(x > 0, 1, 0))} %>%
    {ifelse(as.matrix(.) > 0, 1, 0)} %>%
    {as.data.frame(.)} %>%
    {dplyr::mutate(.,Orthology_Entry = rownames(.)
    )}
}

#' @title Process Module Components with Comma Handling
#'
#' @description Processes a vector of KOs, applying different handling methods depending on whether
#'              they contain commas or not. Useful for processing complex pathway definitions with
#'              alternative KOs.
#'
#' @param KO_vector Character vector of KO identifiers to process
#' @param module_abundance Data frame containing KO abundance data
#' @param process_step_comma Function to handle comma-separated KOs (alternative forms)
#' @param process_step_direct Function to handle individual KOs
#' @param aggregrate_rowname Base name for row aggregation (default: 'step_1')
#' @param step_count Counter for processing steps (default: 1)
#' @param comma_scale_method Scaling method for comma-separated KOs ("sum" or "max")
#'
#' @return List containing:
#'         - abundance_table: Processed abundance Data frame
#'         - step_count: Updated step counter
#'         - abundance_log: log
#' @export
process_module_loop_comma <- function(KO_vector,
                                      module_abundance,
                                      process_step_comma,
                                      process_step_direct,
                                      aggregrate_rowname = 'step_1',
                                      step_count = 1,
                                      comma_scale_method) {
  # Initialize empty data frames for results
  abundance_table = data.frame()
  abundance_table.tmp = data.frame()
  log_messages = list()

  # Process each KO in the input vector
  for (KOs in KO_vector) {
    # Check if KO string contains commas (alternative KOs)
    has_comma <- base::grepl(",", KOs)

    if (has_comma) {
      # Process comma-separated KOs using specified method
      result = process_step_comma(
        module_abundance,
        KOs,
        aggregrate_rowname = paste0(aggregrate_rowname, '_', step_count),
        step_count,
        comma_scale_method
      )
      abundance_table.tmp = result[['abundance_table']]
      step_count = result[['step_count']]
      log_messages.tmp = result[['abundance_log']]
    }
    else {
      # Process single KO directly
      abundance_list = process_step_direct(module_abundance, KOs)
      abundance_table.tmp = abundance_list[["abundance_table"]]
      log_messages.tmp = abundance_list[["abundance_log"]]
    }
    # Combine results while removing duplicates
    abundance_table <- rbind(abundance_table, abundance_table.tmp) %>% unique(.)
    log_messages = c(log_messages, log_messages.tmp)
  }
  return(list(abundance_table = abundance_table, step_count = step_count, abundance_log = log_messages))
}

#' @title Process Module Steps with Complex KO String Handling
#'
#' @description Processes a KO string containing various combinations of KOs separated by
#'              different operators (commas, plus signs, or spaces). Handles complex pathway
#'              definitions with multiple types of relationships between KOs.
#'
#' @param module_abundance Data frame containing KO abundance data
#' @param KO_string String representation of KO relationships (default: "K03388,K03389+K03390+K14083,K14126+K14127,K14128")
#' @param aggregrate_rowname Base name for row aggregation (default: 'bracket_1')
#' @param step_count Counter for processing steps (default: 1)
#' @param plus_scale_method Scaling method for plus-separated KOs ("mean", "min", or "max")
#' @param comma_scale_method Scaling method for comma-separated KOs ("sum" or "max")
#'
#' @return List containing:
#'         - abundance_table: Processed abundance values
#'         - step_count: Updated step counter
#'         - abundance_log: log
#' @export
process_module_step <- function(module_abundance,
                                KO_string = "K03388,K03389+K03390+K14083,K14126+K14127,K14128",
                                aggregrate_rowname = 'bracket_1',
                                step_count = 1,
                                plus_scale_method,
                                comma_scale_method) {
  # Remove outer brackets if present
  KO_string = remove_outer_brackets(KO_string)

  # Check for different types of separators in the KO string
  has_comma <- base::grepl(",", KO_string)
  has_plus <- base::grepl("\\+", KO_string)
  has_space <- base::grepl(" ", KO_string)
  abundance_log.tmp = list()

  # Case 1: String contains plus, comma, and space (most complex case)
  # (K14126+K14128,K22516+K00125 K00126 K22516,K14126 K22516+K00125)
  if ((has_plus && has_comma && has_space)) {
    # Split by space first, then handle each component
    # (K14126+K14128,K22516+K00125 K00126) to (K14126+K14128,K22516+K00125) (K00126)
    KO_vector <- base::strsplit(KO_string, " ")[[1]]
    loop_plu_comma_list <- process_module_loop_plu_comma(
      KO_vector,
      module_abundance,
      process_step_plus,
      process_step_comma,
      process_step_direct,
      aggregrate_rowname,
      step_count,
      plus_scale_method,
      comma_scale_method
    )
    abundance_log.tmp = loop_plu_comma_list[['abundance_log']]
    KO_scale = paste(rownames(loop_plu_comma_list[['abundance_table']]), collapse = '+')
    result_list <- process_step_plus(
      loop_plu_comma_list[['abundance_table']],
      KO_scale,
      aggregrate_rowname,
      loop_plu_comma_list[['step_count']],
      plus_scale_method
    )
  }
  # Case 2: String contains both plus and comma (but no space)
  else if (has_plus && has_comma) {
    # Split by comma first, then handle each component
    KO_vector <- base::strsplit(KO_string, ",")[[1]]
    loop_plus_list <- process_module_loop_plus(
      KO_vector,
      module_abundance,
      process_step_plus,
      process_step_direct,
      aggregrate_rowname,
      step_count,
      plus_scale_method
    )
    abundance_log.tmp = loop_plus_list[['abundance_log']]
    KO_scale = paste(rownames(loop_plus_list[['abundance_table']]), collapse = ',')
    result_list <- process_step_comma(
      loop_plus_list[['abundance_table']],
      KO_scale,
      aggregrate_rowname,
      loop_plus_list[['step_count']],
      comma_scale_method
    )
  }
  # Case 3: String contains plus and space (but no comma)
  else if (has_plus && has_space) {
    # Split by space first, then handle each component
    KO_vector <- base::strsplit(KO_string, " ")[[1]]
    loop_plus_list <- process_module_loop_plus(
      KO_vector,
      module_abundance,
      process_step_plus,
      process_step_direct,
      aggregrate_rowname,
      step_count,
      plus_scale_method
    )
    abundance_log.tmp = loop_plus_list[['abundance_log']]
    KO_scale = paste(rownames(loop_plus_list[['abundance_table']]), collapse = ' ')
    result_list <- process_step_space(
      loop_plus_list[['abundance_table']],
      KO_scale,
      aggregrate_rowname,
      loop_plus_list[['step_count']]
    )
  }
  # Case 4: String contains comma and space (but no plus)
  else if (has_comma && has_space) {
    # Split by space first, then handle each component
    KO_vector <- base::strsplit(KO_string, " ")[[1]]
    loop_comma_list <- process_module_loop_comma(
      KO_vector,
      module_abundance,
      process_step_comma,
      process_step_direct,
      aggregrate_rowname,
      step_count,
      comma_scale_method
    )
    abundance_log.tmp = loop_comma_list[['abundance_log']]
    KO_scale = paste(rownames(loop_comma_list[['abundance_table']]), collapse = ' ')
    result_list <- process_step_space(
      loop_comma_list[['abundance_table']],
      KO_scale,
      aggregrate_rowname,
      loop_comma_list[['step_count']]
    )
  }
  # Case 5: String contains only commas (K14126,K22516)
  else if (has_comma) {
    result_list <- process_step_comma(
      module_abundance,
      KO_string,
      aggregrate_rowname,
      step_count,
      comma_scale_method
    )
  }
  # Case 6: String contains only plus signs (K14126+K14128)
  else if (has_plus) {
    result_list <- process_step_plus(
      module_abundance,
      KO_string,
      aggregrate_rowname,
      step_count,
      plus_scale_method
    )
  }
  # Case 7: String contains only spaces (K14126 K22516)
  else if (has_space) {
    result_list <- process_step_space(
      module_abundance,
      KO_string,
      aggregrate_rowname,
      step_count
    )
  }
  # Case 8: Single KO with no separators (K14126)
  else {
    result = process_step_direct(module_abundance, KO_string)
    abundance_table = result[['abundance_table']]
    abundance_log = result[['abundance_log']]
    result_list = list(abundance_table = abundance_table, step_count = step_count, abundance_log = abundance_log)
    #stop("Please check Module Definition!")
  }
  result_list = list(abundance_table = result_list[['abundance_table']],
                     step_count = result_list[['step_count']],
                     abundance_log = c(abundance_log.tmp ,result_list[['abundance_log']]))
  return(result_list)
}

#' @title Merge Module Information with Module Table
#'
#' @description Merges pathway information with a module table to create a sample-by-module matrix
#'              with proper module names. Ensures all modules are represented in the output.
#' @param pathway_infor Data frame containing pathway information, see examples.
#' @param module_table Data frame containing module data with:
#'                     - Module_Entry: Matching module identifiers
#'                     - Orthology_Entry: KO identifiers
#'                     - Definition: Module definitions
#'                     - Sample columns with abundance values
#'
#' @return A data frame where:
#'         - Rows are module names (from Module_Name)
#'         - Columns are samples
#'         - All modules from pathway_infor are represented
#'         - Original row names are replaced with descriptive module names
#' @export
merge_module_name <- function(pathway_infor, module_table) {
  module_infor <- pathway_infor %>%
    dplyr::select(Module_Entry, Module_Name) %>%
    {unique(.)}
  Module_Sample = module_table %>%
    base::merge(., module_infor, all.x = T, by = 'Module_Entry') %>%
    {dplyr::select(., -c(Module_Entry,Orthology_Entry,Definition))} %>%
    {
      rownames(.) = (.$Module_Name)
      (.)
    } %>%
    {dplyr::select(., -Module_Name)} %>%
    {add_rows_if_not_exists(., add_rows = unique(module_infor$Module_Name))}
  return(Module_Sample)
}

#' @title Process Module Structure Data
#'
#' @description Filters and merges pathway information with sample KO data for a specific module.
#'              Returns a merged dataframe containing KO abundance data and pathway definitions.
#' @param pathway_infor Data frame containing pathway information, see examples.
#' @param Sample_KO Dataframe containing KO abundance data with KO IDs as row names
#' @param module Character string of the module ID to process (e.g. "M00563")
#'
#' @return A merged dataframe containing:
#'         - KO abundance data for the specified module
#'         - Corresponding pathway information
#'         - Empty dataframe if no matching KOs found
#' @export
process_module_structure <- function(pathway_infor, Sample_KO, module) {
  # Filter pathway information for the specified module
  each_pathway_infor = pathway_infor %>%
    {.[(.$Module_Entry) %in% module, c("Orthology_Entry","Module_Entry","Definition"),drop = F]} %>%
    {unique(.)}

  # Filter sample KO data for orthology entries in the module
  sub_Sample_KO = Sample_KO %>%
    {.[rownames(.) %in% unique(each_pathway_infor$Orthology_Entry),,drop = F]}

  # Check if any matching KOs were found
  if (length(sub_Sample_KO$Orthology_Entry)>0){
    # Merge KO data with pathway information
    sub_Sample_KO_pathway = sub_Sample_KO %>%
      base::merge(., each_pathway_infor, by = 'Orthology_Entry', all.x = T) %>%
      {
        rownames(.) = .$Orthology_Entry
        (.)
      }
  } else {
    sub_Sample_KO_pathway = data.frame()
  }
  return(sub_Sample_KO_pathway)
}

#' @title Process Module Components with Plus and Comma Handling
#'
#' @description Processes a vector of KOs, applying different handling methods depending on whether
#'              they contain plus signs, commas, or both. Handles complex pathway definitions with
#'              both required components (plus-separated) and alternative forms (comma-separated).
#'
#' @param KO_vector Character vector of KO identifiers to process
#' @param module_abundance Data frame containing KO abundance data
#' @param process_step_plus Function to handle plus-separated KOs (required components)
#' @param process_step_comma Function to handle comma-separated KOs (alternative forms)
#' @param process_step_direct Function to handle individual KOs
#' @param aggregrate_rowname Base name for row aggregation (default: 'step_1')
#' @param step_count Counter for processing steps (default: 1)
#' @param plus_scale_method Scaling method for plus-separated KOs ("mean", "min", or "max")
#' @param comma_scale_method Scaling method for comma-separated KOs ("sum" or "max")
#'
#' @return List containing:
#'         - abundance_table: Processed abundance values
#'         - step_count: Updated step counter
#'         - abundance_log: log
#' @export
process_module_loop_plu_comma <- function(KO_vector,
                                          module_abundance,
                                          process_step_plus,
                                          process_step_comma,
                                          process_step_direct,
                                          aggregrate_rowname = 'step_1',
                                          step_count = 1,
                                          plus_scale_method,
                                          comma_scale_method) {
  # Initialize empty data frames for results
  abundance_table = data.frame()
  abundance_table.tmp = data.frame()
  abundance_log <- list()
  # Process each KO in the input vector
  for (KOs in KO_vector) {
    # Check for presence of operators
    has_comma <- base::grepl(",", KOs)
    has_plus <- base::grepl("\\+", KOs)

    # Case 1: Contains both '+' and ',' (e.g., "K14126+K14128,K22516+K00125,K00126")
    if (has_plus && has_comma) {
      # First process plus-separated components, then comma-separated alternatives
      KO_subvector <- base::strsplit(KOs, ",")[[1]]
      loop_plus_result <- process_module_loop_plus(
        KO_subvector,
        module_abundance,
        process_step_plus,
        process_step_direct,
        aggregrate_rowname,
        step_count,
        plus_scale_method
      )

      KO_scale <- paste(rownames(loop_plus_result[['abundance_table']]), collapse = ',')
      step_count <- loop_plus_result[['step_count']]
      abundance_log.tmp <- loop_plus_result[['abundance_log']]

      comma_result <- process_step_comma(
        loop_plus_result[['abundance_table']],
        KO_scale,
        aggregrate_rowname = paste0(aggregrate_rowname, '_', step_count),
        step_count,
        comma_scale_method
      )

      abundance_table.tmp <- comma_result[['abundance_table']]
      step_count <- comma_result[['step_count']]
      abundance_log.tmp <- c(abundance_log.tmp, comma_result[['abundance_log']])
    }
    # Case 2: Contains only commas (alternative forms, e.g., "K22516,K14126")
    else if (has_comma) {
      comma_result <- process_step_comma(
        module_abundance,
        KOs,
        aggregrate_rowname = paste0(aggregrate_rowname, '_', step_count),
        step_count,
        comma_scale_method
      )
      abundance_table.tmp <- comma_result[['abundance_table']]
      step_count <- comma_result[['step_count']]
      abundance_log.tmp <- comma_result[['abundance_log']]
    }
    # Case 3: Contains only plus signs (required components, e.g., "K22516+K00125")
    else if (has_plus) {
      plus_result <- process_step_plus(
        module_abundance,
        KOs,
        aggregrate_rowname = paste0(aggregrate_rowname, '_', step_count),
        step_count,
        plus_scale_method
      )
      abundance_table.tmp <- plus_result[['abundance_table']]
      step_count <- plus_result[['step_count']]
      abundance_log.tmp <- plus_result[['abundance_log']]
    }
    # Case 4: Single KO with no operators
    else {
      direct_result <- process_step_direct(module_abundance, KOs)
      abundance_table.tmp <- direct_result[['abundance_table']]
      abundance_log.tmp <- direct_result[['abundance_log']]
    }

    # Combine results while removing duplicates
    abundance_table <- rbind(abundance_table, abundance_table.tmp) %>% unique(.)
    abundance_log = c(abundance_log, abundance_log.tmp)
  }

  return(list(abundance_table = abundance_table, step_count = step_count, abundance_log = abundance_log))
}

#' @title Read and process Pathway information dataframe
#'
#' @description This function reads a tab-delimited file containing KEGG pathway information,
#' performs data validation and cleaning, and returns a processed data frame.
#'
#' @param in_KO_pathway_ref Character string specifying the path to the input file.
#'        The file should be a tab-delimited text file containing KEGG pathway information,
#'        with a header row and at least one column named "Module_Entry".
#'
#' @return A list with log and a data frame containing the processed pathway information after removing
#'         empty/NA entries. The data frame will have the same columns as the input file.
#'         Returns NULL if the file cannot be read.
#'
#' @export
read_and_process_pathway_infor <- function(in_KO_pathway_ref) {
  # Use tryCatch function to handle potential errors
  tryCatch({
    # Read the file
    #pathway_infor = read.csv(in_KO_pathway_ref, sep='\t',header = TRUE, check.names = F)
    pathway_infor = in_KO_pathway_ref

    timestamp <- function() format(Sys.time(), "[%Y-%m-%d %H:%M:%S]")
    log_entry <- function(msg) {
      paste0(timestamp(), " ", msg)
    }

    # Check if the file was successfully imported
    if (!is.null(pathway_infor)) {
      message(paste0('[',format(Sys.time(), "%Y-%m-%d %H:%M:%S"),'] Pathway information dataframe successfully imported.'))
      pathway_log <- list(
          log_entry("Pathway information dataframe successfully imported.")
        )
    } else {
      pathway_log <- list(
          log_entry("*** Pathway information dataframe is null, please check the input file path and format ***")
        )
      stop(paste0('[',format(Sys.time(), "%Y-%m-%d %H:%M:%S"),'] *** Pathway information dataframe is null, please check the input file path and format ***\n'))
    }
    # 删除列a中含有NA的行
    pathway_infor <- pathway_infor[!(pathway_infor$Module_Entry %in% 'NA'), , drop = F]
    pathway_infor <- pathway_infor[!(pathway_infor$Module_Entry %in% ''), , drop = F]
    pathway_infor <- pathway_infor[!is.na(pathway_infor$Module_Entry), , drop = F]

    Module_list <- unique(pathway_infor$Module_Entry)
    if (length(Module_list) > 0) {
      message(paste0('[',format(Sys.time(), "%Y-%m-%d %H:%M:%S"),'] There are ',length(Module_list),' Modules in the Pathway information dataframe.'))
      pathway_log <- c(pathway_log, list(
          log_entry(paste0('There are ',length(Module_list),' Modules in the Pathway information dataframe: ', paste(Module_list, collapse = ' ')))
        ))
    } else {
      pathway_log <- c(pathway_log, list(
          log_entry('*** No Module detected, please check the input file path and format ***')
        ))
      stop(paste0('[',format(Sys.time(), "%Y-%m-%d %H:%M:%S"),'] *** No Module detected, please check the input file path and format ***\n'))
    }
    return(list(data = pathway_infor, log = pathway_log))
  }, error = function(e) {
    stop(paste0('[',format(Sys.time(), "%Y-%m-%d %H:%M:%S"),'] *** Pathway information dataframe import failed, please check the input file path and format ***\n'))
  })
}

#' @title Create and Export Pathway-Specific Module Sample Files
#'
#' @description Processes pathway information and module sample data to create and export
#'              individual pathway-specific files containing scaled module data.
#'
#' @param pathway_infor Data frame containing pathway information, see examples.
#' @param Module_Sample_scale Data frame containing scaled module sample data with
#'                            module names as row names
#' @param out_DIR_Module_Sample_by_pathway Character string specifying output directory
#' @param plus_scale_method Scaling method for plus-separated KOs ("mean", "min", or "max")
#' @param comma_scale_method Scaling method for comma-separated KOs ("sum" or "max")
#'
#' @return None (writes files to disk)
#' @export
create_sub_module_sample <- function(pathway_infor, Module_Sample_scale, out_DIR_Module_Sample_by_pathway, plus_scale_method, comma_scale_method) {
  for (pathway in unique(pathway_infor$Level_2)) {
    message(paste0('[', format(Sys.time(), "%Y-%m-%d %H:%M:%S"), '] Output Pathway: ',unique(pathway)))
    pathway_scale <- gsub(' ', '_', pathway)
    out_file <- base::file.path(out_DIR_Module_Sample_by_pathway, paste0(pathway_scale,".tsv"))

    each_pathway_infor <- pathway_infor %>%
      {.[(.$Level_2) %in% pathway, c('Module_Name', 'Module_Entry'),drop = F]} %>%
      unique()
    sub_Module_Sample_out = Module_Sample_scale %>%
      {as.data.frame(.)[rownames(.) %in% unique(each_pathway_infor$Module_Name), , drop = FALSE]} %>%
      {tibble::rownames_to_column(., var = "Module_Name")}

    data.table::fwrite(sub_Module_Sample_out, file = out_file, sep = "\t", quote = FALSE)
  }
}

#' @title Add Missing Rows to a Data Frame
#'
#' @description This function checks if specified rows exist in a data frame, and if not,
#'              adds them with all values set to 0. Useful for ensuring consistent KO
#'              representation across samples.
#'
#' @param module_abundance A data frame with row names representing KO identifiers
#'                         (e.g., K numbers) and numeric abundance values
#' @param add_rows A character vector of row names (KO identifiers) that should be
#'                 present in the output. Defaults to c("K14126","K14128","K14127")
#'
#' @return The original data frame with additional rows (if any were missing) where
#'         all values are set to 0. Row and column names are preserved.
#' @export
add_rows_if_not_exists <- function(module_abundance, add_rows = c("K14126","K14128","K14127")) {
  missing_rows <- base::setdiff(add_rows, rownames(module_abundance))
  absent_genome_KOs <- data.frame(
    matrix(0,
           nrow = length(missing_rows),
           ncol = ncol(module_abundance))
  )
  colnames(absent_genome_KOs) <- colnames(module_abundance)
  rownames(absent_genome_KOs) <- missing_rows
  result <- rbind(module_abundance, absent_genome_KOs)
  return(result)
}

#' @title Escape Special Characters in a String
#'
#' @description Escapes all specified special characters in a string by adding a backslash before them.
#'              This is particularly useful for preparing strings for use in regular expressions or
#'              other contexts where special characters need to be treated as literals.
#'
#' @param s A character string to be processed
#'
#' @return A new string with all specified special characters escaped with backslashes
#' @export
escape_special_chars <- function(s) {
  # Define all special characters that need to be escaped
  special_chars <- c("(", ")", "+", "-")

  # Escape each special character by adding a backslash before it
  for (char in special_chars) {
    # Use fixed=TRUE for literal matching (faster than regex)
    # paste0 creates the escape sequence (e.g., "(" becomes "\\(")
    s <- gsub(char, paste0("\\", char), s, fixed = TRUE)
  }

  return(s)
}

#' @title Process Space-Separated KOs with Mean Calculation
#'
#' @description Handles space-separated KOs by calculating the mean abundance across all specified KOs.
#'              Processes multiple KOs separated by spaces and aggregates them into a single row.
#'              Note: For mean calculation, uses the sum of KO abundances divided by total number of KOs,
#'              including those with zero abundance in all samples.
#'
#' @param module_abundance Data frame containing KO abundance data with required columns:
#'                         Orthology_Entry, Module_Entry, Definition
#' @param KOs Character string of space-separated KO IDs (default: "K14126 K14127 K14128")
#' @param aggregrate_rowname Base name for row aggregation (default: 'step_1')
#' @param step_count Processing step counter (default: 1)
#'
#' @return List containing:
#'         - abundance_table: Processed data with mean values
#'         - step_count: Updated step counter
#'         - abundance_log: log
#' @export
process_step_space <- function(module_abundance, KOs = c("K14126 K14127 K14128"),
                               aggregrate_rowname, step_count = 1) {
  # Process space-separated KOs (does not enter loop, no new_step_name added)
  # Uses mean calculation similar to plus-separated KOs
  KOs_scale <- base::strsplit(KOs, " ")[[1]]
  #cat(paste0('\t\tRunning KOs space: ', aggregrate_rowname, " = ", KOs_scale, '\n'))
  log_messages <- list(paste0('[',format(Sys.time(), "%Y-%m-%d %H:%M:%S"),']','    ','Running KOs space: ', aggregrate_rowname, " = ", KOs_scale))
  # Prepare and process abundance data
  abundance_table = module_abundance %>%
    {
      rownames(.) = (.$Orthology_Entry)
      (.)
    } %>%
    {dplyr::select(., -c(Orthology_Entry, Module_Entry, Definition))} %>%
    {add_rows_if_not_exists(., add_rows = KOs_scale)} %>%
    {.[rownames(.) %in% KOs_scale, , drop = F]} %>%
    {t(colSums(.)/length(KOs_scale))} %>%  # Calculate mean across all KOs
    {as.data.frame((.), row.names = aggregrate_rowname)} %>%
    {dplyr::mutate(.,
                   Orthology_Entry = rownames(.),
                   Module_Entry = unique(module_abundance$Module_Entry),
                   Definition = unique(module_abundance$Definition))}

  step_count = step_count + 1
  return(list(abundance_table = abundance_table, step_count = step_count, abundance_log = log_messages))
}

#' KEGG Orthology (KO) Pathway Information Dataset
#'
#' A comprehensive dataset mapping KEGG Orthology (KO) entries to metabolic pathways,
#' including module hierarchy, definitions, and enzyme annotations.
#'
#' @format A data frame with 3846 rows (KO entries) and 10 variables:
#' \describe{
#'   \item{Orthology_Entry}{Character. KEGG Orthology ID (e.g., "K00844").}
#'   \item{Module_Type}{Character. Type of metabolic module (e.g., "Pathway modules").}
#'   \item{Level_2}{Character. Broad metabolic category (e.g., "Carbohydrate metabolism").}
#'   \item{Level_3}{Character. Specific metabolic subcategory (e.g., "Central carbohydrate metabolism").}
#'   \item{Module_Entry}{Character. KEGG Module ID (e.g., "M00001").}
#'   \item{Module_Name}{Character. Full name of the metabolic module (e.g., "Glycolysis (Embden-Meyerhof pathway), glucose => pyruvate").}
#'   \item{Definition}{Character. KO composition of the module, truncated in display (e.g., "(K00844,K12407,...)").}
#'   \item{Orthology_Symbol}{Character. Short symbol for the KO (e.g., "HK" for hexokinase).}
#'   \item{Orthology_Name}{Character. Full enzyme name with EC number (e.g., "hexokinase \code{[EC:2.7.1.1]}").}
#'   \item{KO_Symbol}{Character. Combined KO ID and symbol (e.g., "K00844; HK").}
#' }
#' @export
"KO_pathway_ref"


#' KEGG Orthology (KO) Abundance/Presence Across Microbial Samples or Genomes
#'
#' A test dataset (wide-format) showing the relative abundance of KEGG Orthology (KO) entries
#' across multiple microbial samples. Values represent normalized abundance metrics
#' (e.g., TPM, RPKM, relative percentage, presence/absence).
#'
#' @format A data frame with 2495 rows (KO entries) and 5 variables:
#' \describe{
#'   \item{KO}{Character. KEGG Orthology ID (e.g., "K00001").}
#'   \item{Marinobacter salarius}{Numeric. Abundance in Genome "Marinobacter salarius".}
#'   \item{Pseudooceanicola nanhaiensis}{Numeric. Abundance in Genome "Pseudooceanicola nanhaiensis".}
#'   \item{Alteromonas australica}{Numeric. Abundance in Genome "Alteromonas australica".}
#'   \item{Henriciella pelagia}{Numeric. Abundance in Genome "Henriciella pelagia".}
#' }
#' @export
"KO_Sample_wide"

.reference_ko_data <- function() {
  list(
    ref1 = KO_pathway_ref,
    ref2 = KO_Sample_wide
  )
  invisible(NULL)
}

# globals.R
utils::globalVariables(
  c(
    # Variables from data frames/tables
    "KO_Sample_table",
    "Mean_RA",
    "Orthology_Entry",
    "Module_Name",
    "Module_Entry",
    "Definition",
    "Level_2",
    "Level_3",
    "KO_Sample_wide",
    "KO_pathway_ref",
    "sub_Sample_KO_pathway",
    "."
  )
)

#' @title Process Module Brackets Recursively
#'
#' @description Recursively processes nested brackets in module definitions to calculate
#'              pathway completeness or abundance scores. Handles complex pathway structures
#'              with multiple nesting levels.
#'
#' @param module_abundance Data frame containing KO abundance data for the module
#' @param module_steps_str String representation of module steps/structure
#' @param bracket_count Counter for tracking nested bracket levels
#' @param step_count Counter for tracking processing steps
#' @param module_name Name of the module being processed
#' @param raw_module_steps Original unprocessed module steps string
#' @param plus_scale_method Scaling method for plus-separated KOs ("mean", "min", or "max")
#' @param comma_scale_method Scaling method for comma-separated KOs ("sum" or "max")
#' @param abundance_log A character vector of timestamped log messages
#' @return A list with two components:
#' \itemize{
#'   \item `data`: Data frame containing processed abundance values with:
#'         - Rows for each bracket level and final step
#'         - Consistent sample columns as input
#'   \item `log`: A character vector of timestamped log messages.
#' }
#' @export
process_module_brackets <- function(
    module_abundance = sub_Sample_KO_pathway,
    module_steps_str = module_steps_str,
    bracket_count = 1,
    step_count = 1,
    module_name = 'Module',
    raw_module_steps = module_steps_str,
    plus_scale_method,
    comma_scale_method,
    abundance_log = list()) {

  timestamp <- function() format(Sys.time(), "[%Y-%m-%d %H:%M:%S]")
  log_entry <- function(msg) {
    paste0(timestamp(), " ", msg)
  }
  original_module_steps = raw_module_steps
  # Extract the innermost brackets
  brackets <- extract_inner_brackets(module_steps_str)
  # If no brackets are found, return the original string
  if (length(brackets) == 0) {
    #cat('\n\tStart processing final step...\n')
    #cat(paste0('\t\tAnalyzing ',module_name,': ',module_steps_str))
    abundance_log <- c(abundance_log, list(
      log_entry("Start processing final step..."),
      log_entry(paste0("Analyzing ", module_name, ": ", module_steps_str))
    ))
    module_step_list = process_module_step(module_abundance,
                                           module_steps_str,
                                           aggregrate_rowname = module_name,
                                           step_count = bracket_count,
                                           plus_scale_method,
                                           comma_scale_method
                                           )
    abundance_log <- c(abundance_log, module_step_list[['abundance_log']])
    return(list(
      data = module_step_list[['abundance_table']],
      log = abundance_log
    ))
  } else {
    #cat(paste0('\n\tNested steps include: ',brackets))
    #cat(paste0('\tStart processing nested steps...\n'))
    abundance_log <- c(abundance_log, list(
      log_entry(paste0("Nested steps include: ")),
      log_entry(paste0('    ',brackets)),
      log_entry("Start processing nested steps...")
    ))
  }
  # Process each pair of brackets
  for (bracket in brackets) {
    bracket_name = paste0(module_name, '_', bracket_count)
    #cat(paste0('\t\tAnalyzing ',bracket_name,': ',bracket))
    #cat(paste0('\t\tAnalyzing ',': ',bracket_count))
    abundance_log <- c(abundance_log, list(
        log_entry(paste0("Analyzing ", bracket_name, ": ", bracket)),
        log_entry(paste0("Bracket level: ", bracket_count))
      ))
    module_steps_str <- stringr::str_replace(module_steps_str, escape_special_chars(bracket), bracket_name)

    module_step_list = process_module_step(module_abundance, bracket, aggregrate_rowname = bracket_name, step_count = 1,
                                 plus_scale_method,comma_scale_method)
    sample_module_bracket = module_step_list[['abundance_table']]
    module_step_list.log = module_step_list[['abundance_log']]
    abundance_log <- c(abundance_log, module_step_list.log)
    module_abundance = rbind(module_abundance, sample_module_bracket) %>% unique(.)
    bracket_count = bracket_count + 1
  }
  return(process_module_brackets(module_abundance = module_abundance,
                                 module_steps_str = module_steps_str,
                                 bracket_count = bracket_count,
                                 step_count = step_count,
                                 module_name = module_name,
                                 raw_module_steps = raw_module_steps,
                                 plus_scale_method = plus_scale_method,
                                 comma_scale_method = comma_scale_method,
                                 abundance_log = abundance_log))
}

#' @title Process Module Components with Plus Sign Handling
#'
#' @description Processes a vector of KOs, applying different handling methods depending on whether
#'              they contain plus signs or not. Handles pathway definitions with required components
#'              (plus-separated KOs representing complex subunits).
#'
#' @param KO_vector Character vector of KO identifiers to process
#' @param module_abundance Data frame containing KO abundance data
#' @param process_step_plus Function to handle plus-separated KOs (required components)
#' @param process_step_direct Function to handle individual KOs
#' @param aggregrate_rowname Base name for row aggregation (default: 'step_1')
#' @param step_count Counter for processing steps (default: 1)
#' @param plus_scale_method Scaling method for plus-separated KOs ("mean", "min", or "max")
#'
#' @return List containing:
#'         - abundance_table: Processed abundance values
#'         - step_count: Updated step counter
#'         - abundance_log: log
#' @export
process_module_loop_plus <- function(KO_vector,
                                     module_abundance,
                                     process_step_plus,
                                     process_step_direct,
                                     aggregrate_rowname = 'step_1',
                                     step_count = 1,
                                     plus_scale_method) {
  # Initialize empty data frames for results
  abundance_table = data.frame()
  abundance_table.tmp = data.frame()
  log_messages = list()

  # Process each KO in the input vector
  for (KOs in KO_vector) {
    # Check if KO string contains plus signs (required components)
    has_plus <- base::grepl("\\+", KOs)

    if (has_plus) {
      # Process plus-separated KOs using specified method
      result <- process_step_plus(
        module_abundance,
        KOs,
        aggregrate_rowname = paste0(aggregrate_rowname, '_', step_count),
        step_count,
        plus_scale_method
      )
      abundance_table.tmp = result[['abundance_table']]
      step_count = result[['step_count']]
      log_messages.tmp = result[['abundance_log']]
    }
    else {
      # Process single KO directly (no plus signs)
      abundance_list = process_step_direct(module_abundance, KOs)
      abundance_table.tmp = abundance_list[["abundance_table"]]
      log_messages.tmp = abundance_list[["abundance_log"]]

    }
    # Combine results while removing duplicates
    abundance_table <- rbind(abundance_table, abundance_table.tmp) %>% unique(.)
    log_messages = c(log_messages, log_messages.tmp)
  }
  return(list(abundance_table = abundance_table, step_count = step_count, abundance_log = log_messages))
}

#' @title Group KO Abundance Data by Module
#' @description Processes KO abundance data to group by metabolic modules, converting
#'              presence/absence data into module-level KO lists. Handles missing KOs
#'              and maintains sample-specific KO profiles.
#' @param pathway_infor Data frame containing pathway information, see examples.
#' @param Sample_KO_abundance Data frame of KO abundances with:
#'                            - Rows as KO identifiers
#'                            - Columns as samples
#'                            - Orthology_Entry column
#' @return A data frame where:
#'         - Rows are module names
#'         - Columns are samples
#'         - Cell values are space-separated lists of present KOs
#'         - Empty strings for modules with no detected KOs
#' @export
group_ko_by_module <- function(pathway_infor, Sample_KO_abundance) {
  replace_values <- function(df) {
    df[] <- lapply(df, function(x) {
      ifelse(x == 1, df$Orthology_Entry, ifelse(x == 0, "", x))
    })
    return(df)
  }
  Sample_ko_group_by_module = Sample_KO_abundance %>%
    {convert_abundance_to_presence(.)} %>%
    {dplyr::select(., -c(Orthology_Entry))} %>%
    {add_rows_if_not_exists(., add_rows = unique(pathway_infor$Orthology_Entry))} %>%
    {dplyr::mutate(., Orthology_Entry = rownames(.))} %>%
    {base::merge(., unique(pathway_infor[, c("Orthology_Entry","Module_Name")]), all.x = T, by = 'Orthology_Entry')} %>%
    {replace_values(.)} %>%
    {dplyr::select(., -Orthology_Entry)}  %>%
    dplyr::group_by(Module_Name) %>%
    dplyr::summarize(dplyr::across(dplyr::everything(), ~paste(.[. != ""], collapse = " "))) %>%
    dplyr::ungroup(.) %>%
    tibble::column_to_rownames(., 'Module_Name')
  return(Sample_ko_group_by_module)
}

#' @title Direct KO Processing Without Special Handling
#'
#' @description Processes KO abundances directly without any special scaling or aggregation.
#'              Simply extracts the specified KOs from the abundance table while maintaining
#'              the original module metadata.
#'
#' @param module_abundance Data frame containing KO abundance data with required columns:
#'                         Orthology_Entry, Module_Entry, Definition
#' @param KOs Character vector of KO IDs to extract (default: c("K14126","K14128","K14127"))
#' @return List containing:
#'         - abundance_table: A subset of the input data frame containing only the specified KOs,
#'         with original module metadata preserved
#'         - abundance_log: log
#' @export
process_step_direct <- function(module_abundance, KOs = c("K14126","K14128","K14127")) {
  # Process KOs directly without any special handling
  #cat(paste0('\t\tRunning direct KOs: ', KOs, '\n'))
  log_messages <- list(paste0('[',format(Sys.time(), "%Y-%m-%d %H:%M:%S"),']','    ','Running KOs direct: ', KOs))
  abundance_table = module_abundance %>%
    {
      rownames(.) = (.$Orthology_Entry)
      (.)
    } %>%
    {dplyr::select(., -c(Orthology_Entry, Module_Entry, Definition))} %>%
    {add_rows_if_not_exists(., add_rows = KOs)} %>%
    {dplyr::mutate(.,
                   Orthology_Entry = rownames(.),
                   Module_Entry = unique(module_abundance$Module_Entry),
                   Definition = unique(module_abundance$Definition)
    )} %>%
    {.[rownames(.) %in% KOs, , drop = F]}
  return(list(abundance_table = abundance_table, abundance_log = log_messages))
}

#' @title Process All Modules in Pathway Information
#'
#' @description Processes all metabolic modules in pathway information, handling each module's structure,
#'              definition, and bracket components. Aggregates results across all modules.
#' @param pathway_infor Data frame containing pathway information, see examples.
#' @param Sample_KO Data frame containing KO (KEGG Orthology) sample data
#' @param plus_scale_method Scaling method for plus-separated KOs ("mean", "min", or "max")
#' @param comma_scale_method Scaling method for comma-separated KOs ("sum" or "max")
#' @param verbose Logical controlling console output:
#'        \itemize{
#'          \item \code{TRUE} (default): Print progress messages
#'          \item \code{FALSE}: Silent mode
#'        }
#' @return A list with two components:
#' \itemize{
#'   \item `data`: A data frame of processed results for all modules, with unique rows to avoid duplicates.
#'   \item `log`: A character vector of timestamped log messages.
#' }
#' @export
process_all_modules <- function(pathway_infor, Sample_KO,
                                plus_scale_method,
                                comma_scale_method,
                                verbose = TRUE) {

  # Initialize empty result dataframe with proper structure
  result <- data.frame()
  Module_log <- list()

  log_message <- function(msg) {
    timestamp <- format(Sys.time(), "[%Y-%m-%d %H:%M:%S]")
    entry <- paste(timestamp, msg)
    if (verbose) message(entry)
    return(entry)  # 返回标准化日志条目
  }

  # Process each module sequentially
  for (each_module in unique(pathway_infor[, 'Module_Entry'])) {

    Module_log <- c(Module_log, log_message(paste("Starting Module:", each_module)))

    # Process module structure to get KO-sample relationships
    sub_Sample_KO_pathway <- process_module_structure(
      pathway_infor = pathway_infor,
      Sample_KO = Sample_KO,
      module = each_module
    )
    # Check if module contains any KOs
    if (dim(sub_Sample_KO_pathway)[2] > 0) {
      # Process module definition string
      module_def_list <- process_module_definition(sub_Sample_KO_pathway)
      module_definition = module_def_list[["definition"]]
      module_definition_log = module_def_list[["log"]]
      Module_log <- c(Module_log, module_definition_log)

      # Process module brackets and get results
      brackets_list <- process_module_brackets(
        module_abundance = sub_Sample_KO_pathway,
        module_steps_str = module_definition,
        bracket_count = 1,
        step_count = 1,
        module_name = each_module,
        raw_module_steps = module_definition,
        plus_scale_method = plus_scale_method,
        comma_scale_method = comma_scale_method,
        abundance_log = list()
      )
      brackets_list.tmp = brackets_list[['data']]
      brackets_list.log = brackets_list[['log']]

      # Log successful processing
      #cat(paste0('\n[', format(Sys.time(), "%Y-%m-%d %H:%M:%S"),
      #          '] Module ', each_module, ' finished: ',
      #          unique(sub_Sample_KO_pathway$Definition), '\n'))
      Module_log <- c(Module_log, brackets_list.log, log_message(paste("Completed Module:", each_module)))

    } else {
      # Log warning for modules with no detected KOs
      #cat(paste0('\n[', format(Sys.time(), "%Y-%m-%d %H:%M:%S"),
      #          '] Warning: Orthology Entry of ', each_module,
      #          ' cannot be detected in this KO-Sample file!\n'))
      Module_log <- c(Module_log, log_message(paste("No KOs detected in module:", each_module)))
      brackets_list.tmp = data.frame()
    }
    # Combine results while removing duplicates
    result <- rbind(result, brackets_list.tmp) %>% unique(.)
  }

  return(list(
    data = result,
    log = Module_log
  ))
}

#' @title Extract Innermost Parentheses Content
#'
#' @description Extracts all text segments enclosed in the innermost level of parentheses from a string.
#'              This is useful for parsing hierarchical or nested parenthetical expressions.
#'
#' @param s A character string to process (can contain multiple parenthetical groups)
#'
#' @return A character vector containing all innermost parenthesized segments
#'         Returns empty character vector if no matches found
#' @export
extract_inner_brackets <- function(s) {
  # Regex explanation:
  # "\\(    " - Matches literal opening parenthesis
  # "[^()]* " - Matches any character except ( or ) zero or more times
  # "\\)    " - Matches literal closing parenthesis
  # This ensures we only match innermost parentheses (no nested cases)
  matches <- stringr::str_extract_all(s, "\\([^()]*\\)")[[1]]

  return(matches)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/process_module_brackets.R
\name{process_module_brackets}
\alias{process_module_brackets}
\title{Process Module Brackets Recursively}
\usage{
process_module_brackets(
  module_abundance = sub_Sample_KO_pathway,
  module_steps_str = module_steps_str,
  bracket_count = 1,
  step_count = 1,
  module_name = "Module",
  raw_module_steps = module_steps_str,
  plus_scale_method,
  comma_scale_method,
  abundance_log = list()
)
}
\arguments{
\item{module_abundance}{Data frame containing KO abundance data for the module}

\item{module_steps_str}{String representation of module steps/structure}

\item{bracket_count}{Counter for tracking nested bracket levels}

\item{step_count}{Counter for tracking processing steps}

\item{module_name}{Name of the module being processed}

\item{raw_module_steps}{Original unprocessed module steps string}

\item{plus_scale_method}{Scaling method for plus-separated KOs ("mean", "min", or "max")}

\item{comma_scale_method}{Scaling method for comma-separated KOs ("sum" or "max")}

\item{abundance_log}{A character vector of timestamped log messages}
}
\value{
A list with two components:
\itemize{
\item \code{data}: Data frame containing processed abundance values with:
- Rows for each bracket level and final step
- Consistent sample columns as input
\item \code{log}: A character vector of timestamped log messages.
}
}
\description{
Recursively processes nested brackets in module definitions to calculate
pathway completeness or abundance scores. Handles complex pathway structures
with multiple nesting levels.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_and_process_pathway_infor.R
\name{read_and_process_pathway_infor}
\alias{read_and_process_pathway_infor}
\title{Read and process Pathway information dataframe}
\usage{
read_and_process_pathway_infor(in_KO_pathway_ref)
}
\arguments{
\item{in_KO_pathway_ref}{Character string specifying the path to the input file.
The file should be a tab-delimited text file containing KEGG pathway information,
with a header row and at least one column named "Module_Entry".}
}
\value{
A list with log and a data frame containing the processed pathway information after removing
empty/NA entries. The data frame will have the same columns as the input file.
Returns NULL if the file cannot be read.
}
\description{
This function reads a tab-delimited file containing KEGG pathway information,
performs data validation and cleaning, and returns a processed data frame.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/process_module_definition.R
\name{process_module_definition}
\alias{process_module_definition}
\title{Process Module Definition String}
\usage{
process_module_definition(sub_Sample_KO_pathway)
}
\arguments{
\item{sub_Sample_KO_pathway}{Data frame containing module definitions in a 'Definition' column}
}
\value{
A list with log and character vector of cleaned module definition strings. The vector contains:
- Removed negative KO indicators
- Simplified parentheses
- Normalized space
}
\description{
Cleans and processes module definition strings by removing various patterns and formatting
elements to extract core KO relationships. Handles special cases like negative KOs and
parenthetical expressions.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/group_ko_by_module.R
\name{group_ko_by_module}
\alias{group_ko_by_module}
\title{Group KO Abundance Data by Module}
\usage{
group_ko_by_module(pathway_infor, Sample_KO_abundance)
}
\arguments{
\item{pathway_infor}{Data frame containing pathway information, see examples.}

\item{Sample_KO_abundance}{Data frame of KO abundances with:
- Rows as KO identifiers
- Columns as samples
- Orthology_Entry column}
}
\value{
A data frame where:
- Rows are module names
- Columns are samples
- Cell values are space-separated lists of present KOs
- Empty strings for modules with no detected KOs
}
\description{
Processes KO abundance data to group by metabolic modules, converting
presence/absence data into module-level KO lists. Handles missing KOs
and maintains sample-specific KO profiles.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/add_rows_if_not_exists.R
\name{add_rows_if_not_exists}
\alias{add_rows_if_not_exists}
\title{Add Missing Rows to a Data Frame}
\usage{
add_rows_if_not_exists(
  module_abundance,
  add_rows = c("K14126", "K14128", "K14127")
)
}
\arguments{
\item{module_abundance}{A data frame with row names representing KO identifiers
(e.g., K numbers) and numeric abundance values}

\item{add_rows}{A character vector of row names (KO identifiers) that should be
present in the output. Defaults to c("K14126","K14128","K14127")}
}
\value{
The original data frame with additional rows (if any were missing) where
all values are set to 0. Row and column names are preserved.
}
\description{
This function checks if specified rows exist in a data frame, and if not,
adds them with all values set to 0. Useful for ensuring consistent KO
representation across samples.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/process_step_comma.R
\name{process_step_comma}
\alias{process_step_comma}
\title{Process Comma-Separated KOs with Specified Scaling Method}
\usage{
process_step_comma(
  module_abundance,
  KOs = c("K14126,K14127,K14128"),
  aggregrate_rowname,
  step_count = 1,
  comma_scale_method
)
}
\arguments{
\item{module_abundance}{Data frame containing KO abundance data with required columns:
Orthology_Entry, Module_Entry, Definition}

\item{KOs}{Character vector of comma-separated KO IDs (default: "K14126,K14127,K14128")}

\item{aggregrate_rowname}{Base name for row aggregation (default: 'step_1')}

\item{step_count}{Processing step counter (default: 1)}

\item{comma_scale_method}{Scaling method for comma-separated KOs ("sum" or "max")}
}
\value{
List containing:
- abundance_table: Processed data with aggregated values
- step_count: Updated step counter
- abundance_log: log
}
\description{
Handles comma-separated KOs by applying the specified scaling method (sum or max).
Processes multiple KOs separated by commas and aggregates them into a single row.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/process_step_space.R
\name{process_step_space}
\alias{process_step_space}
\title{Process Space-Separated KOs with Mean Calculation}
\usage{
process_step_space(
  module_abundance,
  KOs = c("K14126 K14127 K14128"),
  aggregrate_rowname,
  step_count = 1
)
}
\arguments{
\item{module_abundance}{Data frame containing KO abundance data with required columns:
Orthology_Entry, Module_Entry, Definition}

\item{KOs}{Character string of space-separated KO IDs (default: "K14126 K14127 K14128")}

\item{aggregrate_rowname}{Base name for row aggregation (default: 'step_1')}

\item{step_count}{Processing step counter (default: 1)}
}
\value{
List containing:
- abundance_table: Processed data with mean values
- step_count: Updated step counter
- abundance_log: log
}
\description{
Handles space-separated KOs by calculating the mean abundance across all specified KOs.
Processes multiple KOs separated by spaces and aggregates them into a single row.
Note: For mean calculation, uses the sum of KO abundances divided by total number of KOs,
including those with zero abundance in all samples.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mclink.R
\name{mclink}
\alias{mclink}
\title{Metabolic Pathway Coverage Analysis}
\usage{
mclink(
  ref = NULL,
  data = NULL,
  table_feature = "completeness",
  plus_scale_method = "mean",
  comma_scale_method = "max",
  out_dir = NULL,
  split_by_pathway = FALSE
)
}
\arguments{
\item{ref}{Pathway information data frame. When \code{NULL} (default),
uses the built-in \code{\link{KO_pathway_ref}} dataset. Must contain the
same columns as the built-in dataset if providing custom data.}

\item{data}{Sample KO abundance data frame. When \code{NULL} (default),
uses the built-in \code{\link{KO_Sample_wide}} dataset.}

\item{table_feature}{Analysis type, either:
\itemize{
\item "completeness" (binary presence/absence, default)
\item "abundance" (weighted by KO abundance)
}}

\item{plus_scale_method}{Scaling method for plus-separated KOs (K1+K2+...) (Enzyme subunits/Protein complexes):
\itemize{
\item "mean" - Moderate approach (default), calculates average value of all components
\item "min" - Rigorous/conservative estimate, uses lowest value (all components must be present)
\item "max" - Liberal estimate, uses highest value (any component indicates completeness)
}}

\item{comma_scale_method}{Scaling method for comma-separated KOs (K1,K2,...) (Gene isoforms/Alternative pathways):
\itemize{
\item "max" - For completeness analysis (default), any component indicates functional pathway
\item "sum" - For abundance analysis, sums all functionally equivalent variants
}}

\item{out_dir}{Output directory path. If \code{NULL} (default), results are only
returned as R objects without writing files.}

\item{split_by_pathway}{Logical. If \code{TRUE}, splits results by pathway/module.
Requires non-NULL \code{out_dir}. Default: \code{FALSE}.}
}
\value{
A list containing:
\itemize{
\item coverage - Data frame with pathway coverage metrics
\item detected_KOs - List of detected KOs per pathway/module
\item log - log of the analysis process
}

If \code{out_dir} is specified, results are also written as TSV files.
}
\description{
Analyzes metabolic pathway completeness/abundance from (meta)genome KO presence/abundance data.
Can use either built-in KEGG datasets or user-provided data frames. Output includes pathway coverage
metrics and detected KOs in each pathway/module.

The distill analysis of KEGG Module coverage is calculated based on the abundance or presence of KOs
in a given module, as per the KEGG Module Definition. In detail, the coverage of a KEGG Module is
determined by first dividing a set of KOs into distinct steps. The coverage for each step is then
calculated separately, and summarized as the coverage of this KEGG Module. When calculating coverage,
spaces or plus signs connecting KO numbers are interpreted as AND operators, while commas are
interpreted as OR operators. For instance, to calculate the completeness for the
module M00020: “K00058 K00831 (K01079, K02203, K22305)”:
\enumerate{
\item Convert the abundance table of KOs into a 0-1 matrix.
\item Consider K00058 as step 1, K00831 as step 2, and (K01079, K02203, K22305) as step 3.
\item Calculate the \code{maximum} (or \code{minimum}, \code{mean}) value of step 3 (K01079, K02203, K22305).
If presence of any KO indicates completeness, calculate the \code{maximum} value;
If all KOs must be resent for completeness, calculate the \code{minimum} value.
For a moderate approach, calculate the \code{mean} value.
\item Use this value along with the values of step 1 and step 2 to calculate the average value,
representing the completeness of the module M00020.
}
}
\examples{
data(KO_pathway_ref)
data(KO_Sample_wide)
selected_modules <- c("M00176","M00165","M00173","M00374","M00375","M00376","M00377")
KO_pathway_ref_selected <- KO_pathway_ref[KO_pathway_ref$Module_Entry \%in\% selected_modules, ]
mc_list =
  mclink(ref = KO_pathway_ref_selected,
         data = KO_Sample_wide,
         table_feature = "completeness",
         plus_scale_method = "min",
         comma_scale_method = "max")
mc_coverage = mc_list[["coverage"]]
mc_detected_KOs = mc_list[["detected_KOs"]]
mc_log = mc_list[["log"]]
print(head(mc_coverage))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/process_module_structure.R
\name{process_module_structure}
\alias{process_module_structure}
\title{Process Module Structure Data}
\usage{
process_module_structure(pathway_infor, Sample_KO, module)
}
\arguments{
\item{pathway_infor}{Data frame containing pathway information, see examples.}

\item{Sample_KO}{Dataframe containing KO abundance data with KO IDs as row names}

\item{module}{Character string of the module ID to process (e.g. "M00563")}
}
\value{
A merged dataframe containing:
- KO abundance data for the specified module
- Corresponding pathway information
- Empty dataframe if no matching KOs found
}
\description{
Filters and merges pathway information with sample KO data for a specific module.
Returns a merged dataframe containing KO abundance data and pathway definitions.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/process_module_loop_plu_comma.R
\name{process_module_loop_plu_comma}
\alias{process_module_loop_plu_comma}
\title{Process Module Components with Plus and Comma Handling}
\usage{
process_module_loop_plu_comma(
  KO_vector,
  module_abundance,
  process_step_plus,
  process_step_comma,
  process_step_direct,
  aggregrate_rowname = "step_1",
  step_count = 1,
  plus_scale_method,
  comma_scale_method
)
}
\arguments{
\item{KO_vector}{Character vector of KO identifiers to process}

\item{module_abundance}{Data frame containing KO abundance data}

\item{process_step_plus}{Function to handle plus-separated KOs (required components)}

\item{process_step_comma}{Function to handle comma-separated KOs (alternative forms)}

\item{process_step_direct}{Function to handle individual KOs}

\item{aggregrate_rowname}{Base name for row aggregation (default: 'step_1')}

\item{step_count}{Counter for processing steps (default: 1)}

\item{plus_scale_method}{Scaling method for plus-separated KOs ("mean", "min", or "max")}

\item{comma_scale_method}{Scaling method for comma-separated KOs ("sum" or "max")}
}
\value{
List containing:
- abundance_table: Processed abundance values
- step_count: Updated step counter
- abundance_log: log
}
\description{
Processes a vector of KOs, applying different handling methods depending on whether
they contain plus signs, commas, or both. Handles complex pathway definitions with
both required components (plus-separated) and alternative forms (comma-separated).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{KO_Sample_wide}
\alias{KO_Sample_wide}
\title{KEGG Orthology (KO) Abundance/Presence Across Microbial Samples or Genomes}
\format{
A data frame with 2495 rows (KO entries) and 5 variables:
\describe{
\item{KO}{Character. KEGG Orthology ID (e.g., "K00001").}
\item{Marinobacter salarius}{Numeric. Abundance in Genome "Marinobacter salarius".}
\item{Pseudooceanicola nanhaiensis}{Numeric. Abundance in Genome "Pseudooceanicola nanhaiensis".}
\item{Alteromonas australica}{Numeric. Abundance in Genome "Alteromonas australica".}
\item{Henriciella pelagia}{Numeric. Abundance in Genome "Henriciella pelagia".}
}
}
\usage{
KO_Sample_wide
}
\description{
A test dataset (wide-format) showing the relative abundance of KEGG Orthology (KO) entries
across multiple microbial samples. Values represent normalized abundance metrics
(e.g., TPM, RPKM, relative percentage, presence/absence).
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_and_process_KO_table.R
\name{read_and_process_KO_table}
\alias{read_and_process_KO_table}
\title{Read and Process KO Sample Table with Pathway Information}
\usage{
read_and_process_KO_table(in_KO_Sample_wide, pathway_infor)
}
\arguments{
\item{in_KO_Sample_wide}{Character string specifying the path to the input KO sample table file.
Should be a tab-delimited file with KO identifiers as row names.}

\item{pathway_infor}{Data frame containing pathway information, see examples.}
}
\value{
A list with log and a data frame.The data frame contains the filtered sample data, with only rows that match KOs in the
pathway information. Includes an added 'Orthology_Entry' column containing the row names.
}
\description{
This function reads a KO sample table (wide format), validates its contents against
pathway information, and returns a filtered table containing only KOs present in both datasets.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/process_all_pathways.R
\name{process_all_pathways}
\alias{process_all_pathways}
\title{Process All Pathways Analysis}
\usage{
process_all_pathways(
  pathway_infor,
  Module_Sample,
  out_DIR_Module_Sample_by_pathway,
  compare_method = c("log", "avg", "round"),
  plus_scale_method,
  comma_scale_method
)
}
\arguments{
\item{pathway_infor}{Data frame containing pathway information, see examples.}

\item{Module_Sample}{Data frame of module sample data to process}

\item{out_DIR_Module_Sample_by_pathway}{Output directory for pathway-specific results}

\item{compare_method}{Comparison method to use: "log" (log10 transform),
"avg" (average calculation), or "round" (simple rounding)}

\item{plus_scale_method}{Scaling method for plus-separated KOs ("mean", "min", or "max")}

\item{comma_scale_method}{Scaling method for comma-separated KOs ("sum" or "max")}
}
\value{
Main outputs are written to:
- Combined module file (All_modules.*.tsv)
- Pathway-specific files (via create_sub_module_sample)
}
\description{
Processes module sample data across all pathways with specified scaling methods.
Handles different comparison methods and outputs results by pathway.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/remove_outer_brackets.R
\name{remove_outer_brackets}
\alias{remove_outer_brackets}
\title{Remove Outer Parentheses from String}
\usage{
remove_outer_brackets(s)
}
\arguments{
\item{s}{A character string to be processed}
}
\value{
The input string with outer parentheses removed (if they existed),
or the original string if no outer parentheses were found.
}
\description{
This function checks if a string is wrapped in outer parentheses
and removes them if present.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convert_abundance_to_presence.R
\name{convert_abundance_to_presence}
\alias{convert_abundance_to_presence}
\title{Convert Abundance Values to Presence/Absence Indicators}
\usage{
convert_abundance_to_presence(module_abundance)
}
\arguments{
\item{module_abundance}{A data frame containing KO abundance data, must include:
- Rows named by Orthology_Entry (KO identifiers)
- Numeric columns representing sample abundances
- An Orthology_Entry column}
}
\value{
A data frame with:
- Binary values (1 = present, 0 = absent) for each sample
- Original row names preserved in Orthology_Entry column
- Same dimensions as input (excluding the Orthology_Entry column)
}
\description{
Transforms a numeric abundance matrix into a binary presence/absence matrix,
where 1 indicates presence (abundance > 0) and 0 indicates absence.
Preserves row names as Orthology_Entry column in the output.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/process_module_step.R
\name{process_module_step}
\alias{process_module_step}
\title{Process Module Steps with Complex KO String Handling}
\usage{
process_module_step(
  module_abundance,
  KO_string = "K03388,K03389+K03390+K14083,K14126+K14127,K14128",
  aggregrate_rowname = "bracket_1",
  step_count = 1,
  plus_scale_method,
  comma_scale_method
)
}
\arguments{
\item{module_abundance}{Data frame containing KO abundance data}

\item{KO_string}{String representation of KO relationships (default: "K03388,K03389+K03390+K14083,K14126+K14127,K14128")}

\item{aggregrate_rowname}{Base name for row aggregation (default: 'bracket_1')}

\item{step_count}{Counter for processing steps (default: 1)}

\item{plus_scale_method}{Scaling method for plus-separated KOs ("mean", "min", or "max")}

\item{comma_scale_method}{Scaling method for comma-separated KOs ("sum" or "max")}
}
\value{
List containing:
- abundance_table: Processed abundance values
- step_count: Updated step counter
- abundance_log: log
}
\description{
Processes a KO string containing various combinations of KOs separated by
different operators (commas, plus signs, or spaces). Handles complex pathway
definitions with multiple types of relationships between KOs.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/escape_special_chars.R
\name{escape_special_chars}
\alias{escape_special_chars}
\title{Escape Special Characters in a String}
\usage{
escape_special_chars(s)
}
\arguments{
\item{s}{A character string to be processed}
}
\value{
A new string with all specified special characters escaped with backslashes
}
\description{
Escapes all specified special characters in a string by adding a backslash before them.
This is particularly useful for preparing strings for use in regular expressions or
other contexts where special characters need to be treated as literals.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/merge_module_name.R
\name{merge_module_name}
\alias{merge_module_name}
\title{Merge Module Information with Module Table}
\usage{
merge_module_name(pathway_infor, module_table)
}
\arguments{
\item{pathway_infor}{Data frame containing pathway information, see examples.}

\item{module_table}{Data frame containing module data with:
- Module_Entry: Matching module identifiers
- Orthology_Entry: KO identifiers
- Definition: Module definitions
- Sample columns with abundance values}
}
\value{
A data frame where:
- Rows are module names (from Module_Name)
- Columns are samples
- All modules from pathway_infor are represented
- Original row names are replaced with descriptive module names
}
\description{
Merges pathway information with a module table to create a sample-by-module matrix
with proper module names. Ensures all modules are represented in the output.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ata_cal.R
\name{ata_cal}
\alias{ata_cal}
\title{Calculate Log2 Ratio of Sample Values to Row Means}
\usage{
ata_cal(data = KO_Sample_table)
}
\arguments{
\item{data}{A data frame or matrix containing numerical values to be processed.
Rows represent features (e.g., KO terms) and columns represent samples.
Default is KO_Sample_table.}
}
\value{
A data frame of the same dimensions as input, where each value is:
- log2(sample_value/row_mean) when both sample_value and row_mean are non-zero
- 0 when either sample_value or row_mean is zero
The Mean_RA column used for calculations is removed from the output.
}
\description{
This function calculates the log2 ratio of each value in a data frame
to its corresponding row mean. Zero values are preserved as zeros in the output.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{KO_pathway_ref}
\alias{KO_pathway_ref}
\title{KEGG Orthology (KO) Pathway Information Dataset}
\format{
A data frame with 3846 rows (KO entries) and 10 variables:
\describe{
\item{Orthology_Entry}{Character. KEGG Orthology ID (e.g., "K00844").}
\item{Module_Type}{Character. Type of metabolic module (e.g., "Pathway modules").}
\item{Level_2}{Character. Broad metabolic category (e.g., "Carbohydrate metabolism").}
\item{Level_3}{Character. Specific metabolic subcategory (e.g., "Central carbohydrate metabolism").}
\item{Module_Entry}{Character. KEGG Module ID (e.g., "M00001").}
\item{Module_Name}{Character. Full name of the metabolic module (e.g., "Glycolysis (Embden-Meyerhof pathway), glucose => pyruvate").}
\item{Definition}{Character. KO composition of the module, truncated in display (e.g., "(K00844,K12407,...)").}
\item{Orthology_Symbol}{Character. Short symbol for the KO (e.g., "HK" for hexokinase).}
\item{Orthology_Name}{Character. Full enzyme name with EC number (e.g., "hexokinase \code{[EC:2.7.1.1]}").}
\item{KO_Symbol}{Character. Combined KO ID and symbol (e.g., "K00844; HK").}
}
}
\usage{
KO_pathway_ref
}
\description{
A comprehensive dataset mapping KEGG Orthology (KO) entries to metabolic pathways,
including module hierarchy, definitions, and enzyme annotations.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/process_all_modules.R
\name{process_all_modules}
\alias{process_all_modules}
\title{Process All Modules in Pathway Information}
\usage{
process_all_modules(
  pathway_infor,
  Sample_KO,
  plus_scale_method,
  comma_scale_method,
  verbose = TRUE
)
}
\arguments{
\item{pathway_infor}{Data frame containing pathway information, see examples.}

\item{Sample_KO}{Data frame containing KO (KEGG Orthology) sample data}

\item{plus_scale_method}{Scaling method for plus-separated KOs ("mean", "min", or "max")}

\item{comma_scale_method}{Scaling method for comma-separated KOs ("sum" or "max")}

\item{verbose}{Logical controlling console output:
\itemize{
\item \code{TRUE} (default): Print progress messages
\item \code{FALSE}: Silent mode
}}
}
\value{
A list with two components:
\itemize{
\item \code{data}: A data frame of processed results for all modules, with unique rows to avoid duplicates.
\item \code{log}: A character vector of timestamped log messages.
}
}
\description{
Processes all metabolic modules in pathway information, handling each module's structure,
definition, and bracket components. Aggregates results across all modules.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/create_sub_module_sample.R
\name{create_sub_module_sample}
\alias{create_sub_module_sample}
\title{Create and Export Pathway-Specific Module Sample Files}
\usage{
create_sub_module_sample(
  pathway_infor,
  Module_Sample_scale,
  out_DIR_Module_Sample_by_pathway,
  plus_scale_method,
  comma_scale_method
)
}
\arguments{
\item{pathway_infor}{Data frame containing pathway information, see examples.}

\item{Module_Sample_scale}{Data frame containing scaled module sample data with
module names as row names}

\item{out_DIR_Module_Sample_by_pathway}{Character string specifying output directory}

\item{plus_scale_method}{Scaling method for plus-separated KOs ("mean", "min", or "max")}

\item{comma_scale_method}{Scaling method for comma-separated KOs ("sum" or "max")}
}
\value{
None (writes files to disk)
}
\description{
Processes pathway information and module sample data to create and export
individual pathway-specific files containing scaled module data.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extract_inner_brackets.R
\name{extract_inner_brackets}
\alias{extract_inner_brackets}
\title{Extract Innermost Parentheses Content}
\usage{
extract_inner_brackets(s)
}
\arguments{
\item{s}{A character string to process (can contain multiple parenthetical groups)}
}
\value{
A character vector containing all innermost parenthesized segments
Returns empty character vector if no matches found
}
\description{
Extracts all text segments enclosed in the innermost level of parentheses from a string.
This is useful for parsing hierarchical or nested parenthetical expressions.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/process_step_plus.R
\name{process_step_plus}
\alias{process_step_plus}
\title{Process Plus-Separated KOs with Specified Scaling Method}
\usage{
process_step_plus(
  module_abundance,
  KOs = c("K14126+K14127+K14128"),
  aggregrate_rowname,
  step_count = 1,
  plus_scale_method
)
}
\arguments{
\item{module_abundance}{Data frame containing KO abundance data with required columns:
Orthology_Entry, Module_Entry, Definition}

\item{KOs}{Character string of plus-separated KO IDs (default: "K14126+K14127+K14128")}

\item{aggregrate_rowname}{Base name for row aggregation (default: 'step_1')}

\item{step_count}{Processing step counter (default: 1)}

\item{plus_scale_method}{Scaling method for plus-separated KOs ("mean", "min", or "max")}
}
\value{
List containing:
- abundance_table: Processed data with aggregated values
- step_count: Updated step counter
- abundance_log: log
}
\description{
Handles plus-separated KOs by applying the specified scaling method (mean, min, or max).
Processes multiple KOs separated by plus signs and aggregates them into a single row.
Note: For mean calculation, uses the sum of existing KO abundances divided by total
number of KOs (including those with zero abundance in all samples).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/process_step_direct.R
\name{process_step_direct}
\alias{process_step_direct}
\title{Direct KO Processing Without Special Handling}
\usage{
process_step_direct(module_abundance, KOs = c("K14126", "K14128", "K14127"))
}
\arguments{
\item{module_abundance}{Data frame containing KO abundance data with required columns:
Orthology_Entry, Module_Entry, Definition}

\item{KOs}{Character vector of KO IDs to extract (default: c("K14126","K14128","K14127"))}
}
\value{
List containing:
- abundance_table: A subset of the input data frame containing only the specified KOs,
with original module metadata preserved
- abundance_log: log
}
\description{
Processes KO abundances directly without any special scaling or aggregation.
Simply extracts the specified KOs from the abundance table while maintaining
the original module metadata.
}
